/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Map;
import javax.wsdl.WSDLException;
import javax.xml.bind.JAXBException;
import org.ow2.orchestra.bpmn2bpel.exceptions.TransformerException;
import org.ow2.orchestra.util.Misc;
import org.xml.sax.SAXException;


/**
 * @author Guillaume Porcher
 */
public final class BarGenerator {

  public byte[] generateBarStream(final URL bpmnUrl) throws JAXBException, IOException, SAXException, WSDLException {
    final Map<String, byte[]> files = new Transformer().transform(bpmnUrl);

    return Misc.generateJar(files);
  }

  public static void main(final String[] args) {
    if (args == null || args.length != 2) {
      throw new RuntimeException("Must give 2 arguments: bpmn file and bar file");
    }
    if (args[0] == null || args[1] == null) {
      throw new RuntimeException("Arguments should not be null");
    }
    try {
      final File bpmnFile = new File(args[0]);
      final byte[] out = new BarGenerator().generateBarStream(bpmnFile.toURI().toURL());
      Misc.write(out, new File(args[1]));
    }
    catch (final IOException e) {
      throw new TransformerException(e.getMessage(), e);
    }
    catch (final WSDLException e) {
      throw new TransformerException(e.getMessage(), e);
    }
    catch (final SAXException e) {
      throw new TransformerException(e.getMessage(), e);
    }
    catch (final JAXBException e) {
      throw new TransformerException(e.getMessage(), e);
    }

  }
}
