/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel.generators;

import javax.wsdl.Definition;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import org.ow2.orchestra.bpmn2bpel.BpmnMap;
import org.ow2.orchestra.bpmn2bpel.components.FlowComponent;
import org.ow2.orchestra.jaxb.bpel.Flow;
import org.ow2.orchestra.jaxb.bpel.TActivity;
import org.ow2.orchestra.jaxb.bpmn.TFlowNode;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.util.Map;


/**
 * @author Guillaume Porcher
 */
class FlowGenerator {

  private final BpelGenerator bpelGenerator;

  FlowGenerator() {
    this.bpelGenerator = new BpelGenerator();
  }

  /**
   * @param bpelGenerator
   */
  FlowGenerator(final BpelGenerator bpelGenerator) {
    this.bpelGenerator = bpelGenerator;
  }

  /**
   * @param bpmnMap
   * @param flowComponent
   * @param processQName
   * @return
   */
  TActivity createFlowActivity(
          final BpmnMap bpmnMap,
          final FlowComponent flowComponent,
          final QName processQName,
          Map<String, Definition> partnerWsdls) throws JAXBException, IOException, SAXException {
    // create bpel flow
    final Flow flow = new Flow();

    // fill the sequence
    for (final TFlowNode node : flowComponent.getFlowNodes()) {
      final TActivity activity = this.bpelGenerator.createBpelActivity(node, bpmnMap, processQName, partnerWsdls);
      flow.getAssignsAndCompensatesAndCompensateScopes().add(activity);
    }
    return flow;
  }

}
