/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel.transformers;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.namespace.QName;
import org.ow2.orchestra.bpmn2bpel.BpmnMap;
import org.ow2.orchestra.jaxb.bpmn.TEndEvent;
import org.ow2.orchestra.jaxb.bpmn.TFlowNode;
import org.ow2.orchestra.jaxb.bpmn.TParallelGateway;
import org.ow2.orchestra.jaxb.bpmn.TSequenceFlow;
import org.ow2.orchestra.jaxb.bpmn.TStartEvent;
import org.ow2.orchestra.util.Misc;

/**
 * @author Guillaume Porcher
 */
public abstract class AbstractFlowLinkTransformer {

  private static final Logger LOG = Logger.getLogger(AbstractFlowLinkTransformer.class.getName());

  protected TParallelGateway getOrAddEndParallelGateway(final BpmnMap bpmnMap) {
    final TEndEvent endNode = bpmnMap.getEndEvents().values().iterator().next();
    final TSequenceFlow toEndSequenceFlow = bpmnMap.getIncomingSequenceFlows(endNode).get(0);
    final TParallelGateway lastGateway;
    if ((toEndSequenceFlow.getSourceRef() instanceof TParallelGateway)
            && bpmnMap.getOutgoingSequenceFlows((TFlowNode) toEndSequenceFlow.getSourceRef()).size() == 1) {
      lastGateway = (TParallelGateway) toEndSequenceFlow.getSourceRef();
    } else {
      lastGateway = new TParallelGateway();
      lastGateway.setId(Misc.getHumanReadableId("component"));
      bpmnMap.getFlowNodes().put(new QName(lastGateway.getId()), lastGateway);

      Misc.fastDynamicLog(AbstractFlowLinkTransformer.LOG, Level.INFO, "Creating end gateway: %s", lastGateway.getId());

      // create sequenceflow from this gateway to end
      bpmnMap.createSequenceFlow(lastGateway, endNode);
      // last activity now goes to this gateway
      bpmnMap.changeSequenceFlowTarget(toEndSequenceFlow, lastGateway);
    }
    return lastGateway;
  }

  protected TParallelGateway getOrAddStartParallelGateway(final BpmnMap bpmnMap) {
    final TStartEvent startNode = bpmnMap.getStartEvents().values().iterator().next();
    final TSequenceFlow fromStartSequenceFlow = bpmnMap.getOutgoingSequenceFlows(startNode).get(0);
    final TParallelGateway firstGateway;
    if ((fromStartSequenceFlow.getTargetRef() instanceof TParallelGateway)
            && bpmnMap.getIncomingSequenceFlows((TFlowNode) fromStartSequenceFlow.getTargetRef()).size() == 1) {
      firstGateway = (TParallelGateway) fromStartSequenceFlow.getTargetRef();
    } else {
      firstGateway = new TParallelGateway();
      firstGateway.setId(Misc.getHumanReadableId("component"));
      bpmnMap.getFlowNodes().put(new QName(firstGateway.getId()), firstGateway);

      Misc.fastDynamicLog(AbstractFlowLinkTransformer.LOG, Level.INFO, "Creating start gateway: %s", firstGateway.getId());

      // create sequenceflow from start node to this gateway
      bpmnMap.createSequenceFlow(startNode, firstGateway);
      // last activity now goes to this gateway
      bpmnMap.changeSequenceFlowSource(fromStartSequenceFlow, firstGateway);
    }
    return firstGateway;
  }

}
